"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startServer = void 0;
const vscode_languageserver_1 = require("vscode-languageserver");
const node_1 = require("vscode-languageserver/node");
const vscode_languageserver_textdocument_1 = require("vscode-languageserver-textdocument");
const MessageHandler = __importStar(require("./lib/MessageHandler"));
const internals_1 = require("./lib/prisma-schema-wasm/internals");
const packageJson = require('../../package.json'); // eslint-disable-line
function getConnection(options) {
    let connection = options === null || options === void 0 ? void 0 : options.connection;
    if (!connection) {
        connection = process.argv.includes('--stdio')
            ? (0, node_1.createConnection)(process.stdin, process.stdout)
            : (0, node_1.createConnection)(new node_1.IPCMessageReader(process), new node_1.IPCMessageWriter(process));
    }
    return connection;
}
let hasCodeActionLiteralsCapability = false;
let hasConfigurationCapability = false;
/**
 * Starts the language server.
 *
 * @param options Options to customize behavior
 */
function startServer(options) {
    // Source code: https://github.com/microsoft/vscode-languageserver-node/blob/main/server/src/common/server.ts#L1044
    const connection = getConnection(options);
    console.log = connection.console.log.bind(connection.console);
    console.error = connection.console.error.bind(connection.console);
    const documents = new vscode_languageserver_1.TextDocuments(vscode_languageserver_textdocument_1.TextDocument);
    connection.onInitialize((params) => {
        var _a, _b, _c;
        // Logging first...
        connection.console.info(`Default version of Prisma 'prisma-schema-wasm': ${(0, internals_1.getVersion)()}`);
        connection.console.info(
        // eslint-disable-next-line
        `Extension name ${packageJson.name} with version ${packageJson.version}`);
        const prismaEnginesVersion = (0, internals_1.getEnginesVersion)();
        connection.console.info(`Prisma Engines version: ${prismaEnginesVersion}`);
        const prismaCliVersion = (0, internals_1.getCliVersion)();
        connection.console.info(`Prisma CLI version: ${prismaCliVersion}`);
        // ... and then capabilities of the language server
        const capabilities = params.capabilities;
        hasCodeActionLiteralsCapability = Boolean((_b = (_a = capabilities === null || capabilities === void 0 ? void 0 : capabilities.textDocument) === null || _a === void 0 ? void 0 : _a.codeAction) === null || _b === void 0 ? void 0 : _b.codeActionLiteralSupport);
        hasConfigurationCapability = Boolean((_c = capabilities === null || capabilities === void 0 ? void 0 : capabilities.workspace) === null || _c === void 0 ? void 0 : _c.configuration);
        const result = {
            capabilities: {
                definitionProvider: true,
                documentFormattingProvider: true,
                completionProvider: {
                    resolveProvider: true,
                    triggerCharacters: ['@', '"', '.'],
                },
                hoverProvider: true,
                renameProvider: true,
                documentSymbolProvider: true,
            },
        };
        if (hasCodeActionLiteralsCapability) {
            result.capabilities.codeActionProvider = {
                codeActionKinds: [vscode_languageserver_1.CodeActionKind.QuickFix],
            };
        }
        return result;
    });
    connection.onInitialized(() => {
        if (hasConfigurationCapability) {
            // Register for all configuration changes.
            // eslint-disable-next-line @typescript-eslint/no-floating-promises
            connection.client.register(vscode_languageserver_1.DidChangeConfigurationNotification.type, undefined);
        }
    });
    // The global settings, used when the `workspace/configuration` request is not supported by the client or is not set by the user.
    // This does not apply to VS Code, as this client supports this setting.
    // const defaultSettings: LSSettings = {}
    // let globalSettings: LSSettings = defaultSettings // eslint-disable-line
    // Cache the settings of all open documents
    const documentSettings = new Map();
    connection.onDidChangeConfiguration((_change) => {
        connection.console.info('Configuration changed.');
        if (hasConfigurationCapability) {
            // Reset all cached document settings
            documentSettings.clear();
        }
        else {
            // globalSettings = <LSSettings>(change.settings.prisma || defaultSettings) // eslint-disable-line @typescript-eslint/no-unsafe-member-access
        }
        // Revalidate all open prisma schemas
        documents.all().forEach(validateTextDocument); // eslint-disable-line @typescript-eslint/no-misused-promises
    });
    // Only keep settings for open documents
    documents.onDidClose((e) => {
        documentSettings.delete(e.document.uri);
    });
    // function getDocumentSettings(resource: string): Thenable<LSSettings> {
    //   if (!hasConfigurationCapability) {
    //     connection.console.info(
    //       `hasConfigurationCapability === false. Defaults will be used.`,
    //     )
    //     return Promise.resolve(globalSettings)
    //   }
    //   let result = documentSettings.get(resource)
    //   if (!result) {
    //     result = connection.workspace.getConfiguration({
    //       scopeUri: resource,
    //       section: 'prisma',
    //     })
    //     documentSettings.set(resource, result)
    //   }
    //   return result
    // }
    // Note: VS Code strips newline characters from the message
    function showErrorToast(errorMessage) {
        connection.window.showErrorMessage(errorMessage);
    }
    function validateTextDocument(textDocument) {
        const diagnostics = MessageHandler.handleDiagnosticsRequest(textDocument, showErrorToast);
        void connection.sendDiagnostics({ uri: textDocument.uri, diagnostics });
    }
    documents.onDidChangeContent((change) => {
        validateTextDocument(change.document);
    });
    function getDocument(uri) {
        return documents.get(uri);
    }
    connection.onDefinition((params) => {
        const doc = getDocument(params.textDocument.uri);
        if (doc) {
            return MessageHandler.handleDefinitionRequest(doc, params);
        }
    });
    connection.onCompletion((params) => {
        const doc = getDocument(params.textDocument.uri);
        if (doc) {
            return MessageHandler.handleCompletionRequest(params, doc, showErrorToast);
        }
    });
    // This handler resolves additional information for the item selected in the completion list.
    connection.onCompletionResolve((completionItem) => {
        return MessageHandler.handleCompletionResolveRequest(completionItem);
    });
    // Unused now
    // TODO remove or experiment new file watcher
    connection.onDidChangeWatchedFiles(() => {
        // Monitored files have changed in VS Code
        connection.console.log(`Types have changed. Sending request to restart TS Language Server.`);
        // Restart TS Language Server
        void connection.sendNotification('prisma/didChangeWatchedFiles', {});
    });
    connection.onHover((params) => {
        const doc = getDocument(params.textDocument.uri);
        if (doc) {
            return MessageHandler.handleHoverRequest(doc, params);
        }
    });
    connection.onDocumentFormatting((params) => {
        const doc = getDocument(params.textDocument.uri);
        if (doc) {
            return MessageHandler.handleDocumentFormatting(params, doc, showErrorToast);
        }
    });
    connection.onCodeAction((params) => {
        const doc = getDocument(params.textDocument.uri);
        if (doc) {
            return MessageHandler.handleCodeActions(params, doc, showErrorToast);
        }
    });
    connection.onRenameRequest((params) => {
        const doc = getDocument(params.textDocument.uri);
        if (doc) {
            return MessageHandler.handleRenameRequest(params, doc);
        }
    });
    connection.onDocumentSymbol((params) => {
        const doc = getDocument(params.textDocument.uri);
        if (doc) {
            return MessageHandler.handleDocumentSymbol(params, doc);
        }
    });
    // Make the text document manager listen on the connection
    // for open, change and close text document events
    documents.listen(connection);
    connection.listen();
}
exports.startServer = startServer;
//# sourceMappingURL=server.js.map